﻿using System;

using com.timeseries;

using edu.kit.ipd.cdm;
using edu.kit.ipd.cdm.changes;
using edu.kit.ipd.cdm.measure;
using edu.kit.ipd.cdm.util;

using java.lang;

using net.sf.jni4net;
using net.sf.jni4net.adaptors;

using Double = java.lang.Double;
using MeasureResultJava = edu.kit.ipd.cdm.MeasureResult;
using Tuple = edu.kit.ipd.cdm.util.Tuple;

using CompressionParameters = edu.kit.ipd.cdm.util.CompressionParameters;
using DefaultParameters = edu.kit.ipd.cdm.util.DefaultParameters;

namespace ChangeDetectionMassLib
{
    public class ChangeDetectionMass
    {
        private static ChangeDetectionMass instance;
        private bool initialized = false;

        public static ChangeDetectionMass get()
        {
            if (null == instance)
            {
                instance = new ChangeDetectionMass();
            }
            return instance;
        }

        private ChangeDetectionMass()
        {
        }

        public void initialize()
        {
            if (initialized)
            {
                return;
            }

            BridgeSetup setup = new BridgeSetup();
            setup.Debug = false;
            setup.Verbose = false;
            setup.VeryVerbose = false;
            setup.AddAllJarsClassPath(@".\lib\");
            //setup.AddJVMOption("-XX:+HeapDumpOnOutOfMemoryError");
            setup.AddJVMOption("-Xmx4096m");
            // Because MATLAB will reuse this JVM instance we might have to add some of its starting 
            // parameters here. Currently it seems to work without them.
            // -XX:PermSize=32m -Xms64m -XX:NewRatio=3 -XX:MaxPermSize=128m -Xmx384m 
            // -XX:MaxDirectMemorySize=2147400000 -Dsun.java2d.noddraw=true -Dsun.awt.nopixfmt=true
            // -Xshare:off -Djava.ext.dirs=C:\Program Files\MATLAB\R2014b\sys\java\jre\win64\jre\lib\ext 
            // -Xrs -Djava.library.path=C:\Program Files\MATLAB\R2014b\bin\win64;C:\Program Files\MATLAB\R2014b\sys\jxbrowser\win64\lib vfprintf 
            // -XX:ErrorFile=C:\Users\Matthias\AppData\Local\Temp\hs_error_pid2516.log abort 
            // -Duser.language=en -Duser.country=US -Djogl.disable.opengles
            Bridge.CreateJVM(setup);
            Bridge.RegisterAssembly(typeof(CDM).Assembly);
            Bridge.RegisterAssembly(typeof(Change).Assembly);
            Bridge.RegisterAssembly(typeof(ChangeList).Assembly);
            Bridge.RegisterAssembly(typeof(IntervalNormalizationOption).Assembly);
            Bridge.RegisterAssembly(typeof(Tuple).Assembly);
            Bridge.RegisterAssembly(typeof(MeasureResultJava).Assembly);
            Bridge.RegisterAssembly(typeof(TimeSeries).Assembly);
            Bridge.RegisterAssembly(typeof(TimeSeriesPoint).Assembly);

            initialized = true;
        }

        /// <summary>
        /// Calculates the difference of change pooints between original and decompressed data. 
        /// If the change detection agorithm does not provide scores, these arrays may be null.
        /// </summary>
        /// <param name="rawData"></param>
        /// <param name="rawChangePoints"></param>
        /// <param name="rawScores"></param>
        /// <param name="decompData"></param>
        /// <param name="decompChangePoints"></param>
        /// <param name="decompScores"></param>
        /// <returns></returns>
        public MeasureResult calculateMeasure(float[] rawData, int[] rawChangePoints, float[] rawScores, 
            float[] decompData, int[] decompChangePoints, float[] decompScores)
        {
            if (!initialized)
            {
                throw new InvalidOperationException("ChangeDetectionMass must be initialized first.");
            }

            ChangeList originalChanges = new ChangeList();
            for (int i = 0; i < rawChangePoints.Length; i++)
            {
                float value = (null == rawScores) ? 1 : rawScores[rawChangePoints[i]];
                originalChanges.add(new Change(rawChangePoints[i], value));
            }

            ChangeList changedChanges = new ChangeList();
            for (int i = 0; i < decompChangePoints.Length; i++)
            {
                float value = (null == decompScores) ? 1 : decompScores[decompChangePoints[i]];
                changedChanges.add(new Change(decompChangePoints[i], value));
            }

            TimeSeries originalTimeSeries = new TimeSeries(1);
            for (int i = 0; i < rawData.Length; i++)
            {
                originalTimeSeries.addLast(i, new TimeSeriesPoint(new double[] { rawData[i] }));
            }

            TimeSeries changedTimeSeries = new TimeSeries(1);
            for (int i = 0; i < decompData.Length; i++)
            {
                changedTimeSeries.addLast(i, new TimeSeriesPoint(new double[] { decompData[i] }));
            }

            //Tuple interval = new Tuple(new Double(0), new Double(5000));
            double epsilon = 0.0001;
            IntervalNormalizationOption options = new IntervalNormalizationOption(null, null, epsilon);

            MeasureResultJava result = CDM.calculateMeasure(originalTimeSeries, originalChanges, changedTimeSeries, changedChanges, options, false, 0.1, CompressionParameters.standardFinalMeasure());
            return new MeasureResult(result.getNumPC(), result.getNumMISS(), result.getNumFP(), result.getPcError(), result.getMissError(), 
                result.getFpError());
        }
    }
}
