// Dies ist die Haupt-DLL.

#include "stdafx.h"
#include <msclr/marshal.h>
#include "CompressionFrameworkLib.h"

using namespace System;
using namespace msclr::interop;

namespace CompressionFrameworkLib{


void CompressionUtility::calculateAvgAndCI(DynArray<double>* values, double &avg, double &ci){
	double neta = 1.96; // Magic Number .. guys from compression framework never changed it, I'm not sure if it is even used.
	if (values->size() <= 0) {
		//printf("No value");
		return;
	}

	double sum = 0;
	for (int i = 0; i < values->size(); i++){
		sum += values->getAt(i);

	}
	avg = sum / values->size();

	double sqrSum = 0;
	for (int i = 0; i < values->size(); i++){
		sqrSum += (avg - values->getAt(i)) * (avg - values->getAt(i));
	}

	ci = sqrt(sqrSum / values->size()) * neta / sqrt((double)values->size());

	/*if (ci > avg){
	printf("\n%f %f\n", avg, ci);
	for (int i = 0; i < values->size(); i++){
	printf("%f\n",values->getAt(i));
	}
	}*/
}

float CompressionUtility::getCompressionRatio(){
	return compressionratio;
}

// Run single-stream algorithm for a given frequency and threshold
DynArray<DataItem>* CompressionUtility::runAlg(char* algo, double eps, CDataStream* dataStream, Metrics &metrics, float seglength)
{
	char EspValue[10];
	::memset(EspValue, 0, sizeof(char)* 10);
	sprintf(EspValue, "%2.5f", eps);

	// Get config
	char* directory = new char[MAX_PATH];
	::memset(directory, 0, sizeof(char)*MAX_PATH);

	//errorType 0 <=> Absolute
	//errorType 1 <=> Percentage	
	int errorType = 1;

	// Metrics, each element stores measure for one stream
	DynArray<double>* compRatios = new DynArray<double>();
	DynArray<double>* segRatios = new DynArray<double>();
	DynArray<double>* RMSEs = new DynArray<double>();
	DynArray<double>* runTimes = new DynArray<double>();

	char** parameter = new char*[2];
	parameter[0] = "compressD";
	parameter[1] = "approxD";

	// Statistics information of input data
	dataStream->statistic();

	// Run algorithm & measure performance
	Algo* algoObj;

	if (!strcmp(_strupr(algo), "APCA"))
	{
		APCAInput* bucketInput = new APCAInput(dataStream, eps, errorType);
		algoObj = new APCA(bucketInput);
	}
	else if (!strcmp(_strupr(algo), "SF"))
	{
		SlideFiltersInput* slideFiltersInput = new SlideFiltersInput(dataStream, eps, errorType);
		algoObj = new SlideFilters(slideFiltersInput);
	}
	else if (!strcmp(_strupr(algo), "CHEB"))
	{
		CHEBInput* inputData = new CHEBInput(dataStream, eps);
		algoObj = new CHEB(inputData, seglength, dataStream->size());
	}
	else
	{
		algoObj = NULL;
		printf("Unsupported Algorithm");
	}

	__int64  nStart = 0, nEnd = 0, nFreq = 0;
	QueryPerformanceCounter((LARGE_INTEGER *)&nStart);

	algoObj->compress();

	// Running time
	QueryPerformanceCounter((LARGE_INTEGER *)&nEnd);
	QueryPerformanceFrequency((LARGE_INTEGER *)&nFreq);
	double duration = (nEnd - nStart) * 1.0 / nFreq;
	runTimes->add(duration / dataStream->size());

	// Write output to file
	algoObj->getOutput()->getParameter(parameter, 2);

	// Ratios
	RMSEs->add(algoObj->getOutput()->getRMSE());
	compRatios->add(algoObj->getOutput()->getCompressionRatio());
	segRatios->add(algoObj->getOutput()->getUpdateFrequency());

	//Store Compression Ratio
	compressionratio = (float)algoObj->getOutput()->getCompressionRatio();

	// Free memory
	delete dataStream;
//	delete algoObj;
	delete[] parameter;


	// Output metrics
	double compRatioAvg, compRatioCI = 0;
	double segRatioAvg, segRatioCI = 0;
	double rmseAvg, rmseCI = 0;
	double timeAvg, timeCI = 0;

	calculateAvgAndCI(compRatios, compRatioAvg, compRatioCI);
	calculateAvgAndCI(segRatios, segRatioAvg, segRatioCI);
	calculateAvgAndCI(RMSEs, rmseAvg, rmseCI);
	calculateAvgAndCI(runTimes, timeAvg, timeCI);

	metrics.setCompRatio(compRatioAvg, compRatioCI);
	metrics.setSegmentRatio(segRatioAvg, segRatioCI);
	metrics.setRMSE(rmseAvg, rmseCI);
	metrics.setTime(timeAvg, timeCI);


	delete compRatios;
	delete segRatios;
	delete RMSEs;
	delete runTimes;

	return algoObj->getOutput()->getDecompressedDataArray();
}
	
	//Method Called from anywhere
array<float>^  CompressionUtility::getDecompressedData(char* algo, array<float>^ data, float epsilon, float seglength){
		Metrics m;
		//Create Datastream
		CDataStream* dataStream = new CDataStream();
		for (int i = 1; i <= data->Length ; i++){
			DataItem entry;
			entry.value = data[i-1];
			entry.timestamp = i;
			dataStream->add(entry);
		}

		DynArray<DataItem> * decompressedDynArray = runAlg(algo, epsilon, dataStream, m, seglength);

		array<float>^ decompressed = gcnew array<float>(decompressedDynArray->size());

		for (int i = 0; i < decompressedDynArray->size(); i++){
			decompressed[i] = (float)decompressedDynArray->getAt(i).value;
		}
		return decompressed;
	}

	array<float>^ CompressionUtility::getDecompressedAPCAData(array<float>^  data, float epsilon){
		return this->getDecompressedData("APCA", data, epsilon,0.0);
	}
	array<float>^ CompressionUtility::getDecompressedSFData(array<float>^  data, float epsilon){
		return this->getDecompressedData("SF", data, epsilon, 0.0);
	}
	array<float>^ CompressionUtility::getDecompressedCHEBData(array<float>^  data, float epsilon, float seglength){
		return this->getDecompressedData("CHEB", data, epsilon,seglength);
	}

}