﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Globalization;
using Microsoft.VisualBasic.FileIO;

namespace Framework
{
    /// <summary>
    /// A simple importer for CSV files.
    /// </summary>
    class CsvImporter
    {
        /// <summary>
        /// Path to the file to read.
        /// </summary>
        private string filePath;

        /// <summary>
        /// Contains the data. First index is the column, second index the row, because we mostly want to access whole 
        /// columns.
        /// </summary>
        private float[][] data = new float[1][];

        /// <summary>
        /// Is set to true after the file was read.
        /// </summary>
        private bool fileRead = false;

        /// <summary>
        /// Create a new CsvImporter for the specifed file.
        /// </summary>
        /// <param name="filePath">Path to a CSV file to read.</param>
        public CsvImporter(string filePath)
        {
            this.filePath = filePath;
        }

        /// <summary>
        /// Read the file.
        /// </summary>
        public void ReadFile()
        {
            using (TextFieldParser parser = new TextFieldParser(filePath))
            {
                parser.TextFieldType = FieldType.Delimited;
                parser.SetDelimiters(",");
                int row = 0;
                while (!parser.EndOfData)
                {
                    for (int i = 0; i < data.Length; i++)
                    {
                        if (data[i] == null)
                        {
                            data[i] = new float[100];
                        }
                        if (row >= data[i].Length)
                        {
                            Array.Resize<float>(ref data[i], row + 1);
                        }
                    }

                    string[] fields = parser.ReadFields();
                    int column = 0;
                    /*foreach (string field in fields)
                    {
                        if (column >= data.Length)
                        {
                            Array.Resize<float[]>(ref data, column + 1);
                        }
                        data[column][row] = float.Parse(field, CultureInfo.InvariantCulture);
                        column++;
                    }*/
                    data[0][row] = float.Parse(fields[0], CultureInfo.InvariantCulture);
                    row++;
                }
            }

            fileRead = true;
        }

        /// <summary>
        /// Get a specific column of the file after readFile was executed.
        /// </summary>
        /// <param name="columnIndex">Get column columnIndex form the file.</param>
        /// <returns></returns>
        public float[] GetColumn(int columnIndex)
        {
            if (!fileRead)
            {
                throw new InvalidOperationException("File has not been read. Call readFile() first.");
            }

            return data[columnIndex];
        }
    }
}
