﻿using ChangeDetectionMassLib;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace Framework
{
    public partial class GUI_graphs : Form
    {
        private string seriesName = "default";
        public string SeriesName
        {
            get
            {
                return seriesName;
            }
            set
            {
                this.seriesName = value;

                if (rawChart.Series.Count > 0)
                {
                    rawChart.Series[0].Name = value;
                }
                if (decompChart.Series.Count > 0)
                {
                    decompChart.Series[0].Name = value;
                }
            }
        }

        public GUI_graphs()
        {
            InitializeComponent();

            rawChart.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
            rawChart.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
            decompChart.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
            decompChart.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
        }

        public void ClearGraphs()
        {
            rawCp_list.Items.Clear();
            decompCp_list.Items.Clear();
            rawChart.Series.Clear();
            decompChart.Series.Clear();

            zoom_gb.Enabled = false;
            zoom_l.Text = "0 points";
        }

        public void SetRawData(float[] values)
        {
            rawChart.Series.Clear();
            rawChart.Series.Add(seriesName);

            for (int i = 0; i < values.Length; i++)
            {
                rawChart.Series[seriesName].Points.AddXY(i, values[i]);
            }
            rawChart.Series[seriesName].ChartType = SeriesChartType.Line;
            rawChart.Series[seriesName].Color = Color.Red;

            zoom_gb.Enabled = true;
            zoom_l.Text = values.Length + " points";
        }

        public void SetDecompressedData(float[] values)
        {
            decompChart.Series.Clear();
            decompChart.Series.Add(seriesName);

            for (int i = 0; i < values.Length; i++)
            {
                decompChart.Series[seriesName].Points.AddXY(i, values[i]);
            }
            decompChart.Series[seriesName].ChartType = SeriesChartType.Line;
            decompChart.Series[seriesName].Color = Color.Red;
        }

        public void SetRawChangePoints(int[] changePoints)
        {
            string seriesName = "change points";
            if (!rawChart.Series.IsUniqueName(seriesName))
            {
                rawChart.Series.RemoveAt(rawChart.Series.IndexOf(seriesName));
            }
            rawCp_list.Items.Clear();
            rawChart.Series.Add(seriesName);

            for (int i = 0; i < changePoints.Length; i++)
            {
                rawChart.Series[seriesName].Points.AddXY(changePoints[i], Program.rawData.Max());
                rawCp_list.Items.Add(changePoints[i]);
            }
            rawChart.Series[seriesName].ChartType = SeriesChartType.Column;
            rawChart.Series[seriesName].Color = Color.Blue;
        }

        public void SetDecompressedChangePoints(int[] changePoints)
        {
            string seriesName = "change points";
            if (decompChart.Series.FindByName(seriesName) != null)
            {
                decompChart.Series.RemoveAt(decompChart.Series.IndexOf(seriesName));
            }
            decompCp_list.Items.Clear();
            decompChart.Series.Add(seriesName);

            for (int i = 0; i < changePoints.Length; i++)
            {
                decompChart.Series[seriesName].Points.AddXY(changePoints[i], Program.decompData.Max());
                decompCp_list.Items.Add(changePoints[i]);
            }
            decompChart.Series[seriesName].ChartType = SeriesChartType.Column;
            decompChart.Series[seriesName].Color = Color.Blue;
        }

        public void SetCompressionStats(float compressionRatio, float rmse)
        {
            cr_l.Text = Math.Round(compressionRatio, 3).ToString();
            rmse_l.Text = Math.Round(rmse, 3).ToString();
        }

        public void SetMeasureStats(MeasureResult measure, float fitness)
        {
            absPC_l.Text = measure.getNumPC().ToString();
            absMISS_l.Text = measure.getNumMISS().ToString();
            absFP_l.Text = measure.getNumFP().ToString();

            errPC_l.Text = Math.Round(measure.getPcError(), 3).ToString();
            errMISS_l.Text = Math.Round(measure.getMissError(), 3).ToString();
            errFP_l.Text = Math.Round(measure.getFpError(), 3).ToString();
            fitness_l.Text = Math.Round(fitness, 3).ToString();
            errTotal_l.Text = Math.Round(measure.getTotalError(),3).ToString();
            
        }

        private void zoom_tb_Scroll(object sender, EventArgs e)
        {
            double halfStep = rawChart.Series[0].Points.Count * 0.005 * zoom_tb.Value;
            double max = rawChart.Series[0].Points.Count;
            rawChart.ChartAreas[0].AxisX.ScaleView.Zoom(0.0 + halfStep, max - halfStep);
            decompChart.ChartAreas[0].AxisX.ScaleView.Zoom(0.0 + halfStep, max - halfStep);
            zoom_l.Text = (max - halfStep * 2.0).ToString() + " points";
        }

        private void zoomIn_b_Click(object sender, EventArgs e)
        {
            if (zoom_tb.Value >= 90) zoom_tb.Value = 99;
            else zoom_tb.Value = zoom_tb.Value + 10;
            zoom_tb_Scroll(null, null);
        }

        private void zoomOut_b_Click(object sender, EventArgs e)
        {
            if (zoom_tb.Value < 10) zoom_tb.Value = 0;
            else zoom_tb.Value = zoom_tb.Value - 10;
            zoom_tb_Scroll(null, null);
        }

        private void rawChart_AxisViewChanging(object sender, ViewEventArgs e)
        {
            decompChart.ChartAreas[0].AxisX.ScaleView.Zoom(rawChart.ChartAreas[0].AxisX.ScaleView.ViewMinimum, rawChart.ChartAreas[0].AxisX.ScaleView.ViewMaximum);
            double differenz = rawChart.ChartAreas[0].AxisX.ScaleView.ViewMaximum - rawChart.ChartAreas[0].AxisX.ScaleView.ViewMinimum;
            double max = rawChart.Series[0].Points.Count;
            zoom_tb.Value = (((int)(differenz * 100.0 / max) - 1) < 0) ? 0 : (int)((differenz * 100.0 / max) - 1);
            zoom_l.Text = differenz.ToString() + " points";
        }

        private void decompChart_AxisViewChanging(object sender, ViewEventArgs e)
        {
            rawChart.ChartAreas[0].AxisX.ScaleView.Zoom(decompChart.ChartAreas[0].AxisX.ScaleView.ViewMinimum, decompChart.ChartAreas[0].AxisX.ScaleView.ViewMaximum);
            double differenz = rawChart.ChartAreas[0].AxisX.ScaleView.ViewMaximum - rawChart.ChartAreas[0].AxisX.ScaleView.ViewMinimum;
            double max = rawChart.Series[0].Points.Count;
            zoom_tb.Value = (((int)(differenz * 100.0 / max) - 1) < 0) ? 0 : (int)((differenz * 100.0 / max) - 1);
            zoom_l.Text = differenz.ToString() + " points";
        }

        private void GUI_graphs_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            if (e.CloseReason == CloseReason.UserClosing)
            {
                e.Cancel = true;
            }
        }

        private void tableLayoutPanel1_Paint(object sender, PaintEventArgs e)
        {

        }
    }
}
