﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Framework.changedetection;
using CompressionFrameworkLib;
using Framework.compression.algorithms;
using Framework.compression;
using Framework.optimization;
using DeutscheBahn;
using ChangeDetectionMassLib;
using Newtonsoft.Json;
using System.Transactions;
using System.Threading;
using System.Globalization;

namespace Framework
{
    public partial class GUI_main : Form
    {
        GUI_graphs graphsForm;
        private GUI_loading loadingForm;
        Task task = new Task();


        public GUI_main()
        {
            showLoading(true, "Loading CAROL");
            InitializeComponent();
            evo_rb.Checked = true;

            // Initialize jni4net bridge as soon as possible. If it is initialized after the MATLAB
            // Runtime which uses a JVM on its own, it will reuse the MATLAB JVM which does not have 
            // the required jni4net classes in its classpath. Therefore the jni4net JVM must be 
            // initialized first. MATLAB will then reuse the jni4net JVM but it does work.
            ChangeDetectionMass.get().initialize();

            graphsForm = new GUI_graphs();
            showLoading(false);
            //graphsForm.Show();
        }

        private void load_b_Click(object sender, EventArgs e)
        {
            openForm.Title = "Choose Dataset";
            openForm.ShowDialog(this);
            if (openForm.FileName == "") return;

            // reset data objects and GUI//
            Program.rawData = null;
            Program.decompData = null;
            Program.rawChangePointScores = null;
            Program.decompChangePointScores = null;
            Program.rawChangePoints = null;
            Program.decompChangePoints = null;
            graphsForm.ClearGraphs();

            CsvImporter importer = new CsvImporter(openForm.FileName);
            importer.ReadFile();
            Program.rawData = importer.GetColumn(0);

            graphsForm.SeriesName = System.IO.Path.GetFileName(openForm.FileName);
            graphsForm.SetRawData(Program.rawData);
        }

        private void findCP_b_Click(object sender, EventArgs e)
        {
            if (Program.rawData == null) load_b_Click(null, null);
            if (Program.rawData == null) return;

            if (Program.CpAlgo == null)
            {
                MessageBox.Show("Please choose a change detection algorithm!", "No Change Detection Algorithm",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            status_tsl.Text = "Finding change points";
            showProgress(true);
            cpWorker.RunWorkerAsync();
        }

        private void compress_b_Click(object sender, EventArgs e)
        {
            if (Program.rawData == null) load_b_Click(null, null); // if no data is loaded, promt to load it
            if (Program.rawData == null) return;


            if (null == Program.CompAlgo)
            {
                MessageBox.Show("Please choose a compression algorithm!", "No compression algorithem",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            status_tsl.Text = "Compressing data";
            showProgress(true);

            compressWorker.RunWorkerAsync();
        }

        private void comp_rbs_CheckedChanged(object sender, EventArgs e)
        {
            if (apca_rb.Checked)
                Program.CompAlgo = new APCA();
            else if (sf_rb.Checked)
                Program.CompAlgo = new SF();
            else if (cheb_rb.Checked)
                Program.CompAlgo = new CHEB();
            else if (pca_rb.Checked)
                Program.CompAlgo = new PCA();
            else if (wavelet_rb.Checked)
                Program.CompAlgo = new Wavelet();
            else if (initial_rb.Checked)
                Program.CompAlgo = null;

            if (!initial_rb.Checked)
                task.CompressionAlgorithm = Program.CompAlgo.Name;
            else
                task.CompressionAlgorithm = "Find Initial";

            if (Program.CompAlgo == null)
            {
                compParametersGrid.DataSource = null;
                return;
            }

            compParametersGrid.DataSource = Utilities.DictionaryToList(Program.CompAlgo.GetParameters());

            // Grid-Layout //
            compParametersGrid.Columns[0].ReadOnly = true;
            compParametersGrid.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            compParametersGrid.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        }

        private void parametersGrid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                Program.CompAlgo.SetParameters(Utilities.ListToDictionary((List<Parameter>)compParametersGrid.DataSource, Program.CompAlgo.GetParameters()));
            }
            catch (ArgumentOutOfRangeException ex)
            {
                MessageBox.Show("Argument nicht innerhalb der erlaubten Werte:\n" + ex.Message);
            }
        }

        private void cpParametersGrid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                Program.CpAlgo.SetParameters(Utilities.ListToDictionary((List<Parameter>)cpParametersGrid.DataSource, Program.CpAlgo.GetParameters()));
            }
            catch (ArgumentOutOfRangeException ex)
            {
                MessageBox.Show("Argument nicht innerhalb der erlaubten Werte:\n" + ex.Message);
            }
        }

        private void cp_rbs_CheckedChanged(object sender, EventArgs e)
        {
            if (bayes_rb.Checked) Program.CpAlgo = new BayesOnlineCD();
            if (adwin_rb.Checked) Program.CpAlgo = new ADWIN();
            if (kernel_rb.Checked) Program.CpAlgo = new OnlineKernelChangeDetection();
            if (changeFinder_rb.Checked) Program.CpAlgo = new ChangeFinderCD();
            if (eventDetection_rb.Checked) Program.CpAlgo = new EventDetection();
            if (direct_rb.Checked) Program.CpAlgo = new Framework.changedetection.DirectDensity();

            task.CPAlgorithm = Program.CpAlgo.Name;

            if (Program.CpAlgo == null)
            {
                cpParametersGrid.DataSource = null;
                return;
            }

            cpParametersGrid.DataSource = Utilities.DictionaryToList(Program.CpAlgo.GetParameters());
            // Grid-Layout //
            cpParametersGrid.Columns[0].ReadOnly = true;
            cpParametersGrid.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            cpParametersGrid.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;

            setInitialParameters();
        }

        private void cpParametersGrid_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                compress_b_Click(null, null);
            }
        }

        private void compParametersGrid_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                findCP_b_Click(null, null);
            }
        }

        private void optParametersGrid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                Program.OptAlgo.SetParameters(Utilities.ListToDictionary((List<Parameter>)optParametersGrid.DataSource, Program.OptAlgo.GetParameters()));
            }
            catch (ArgumentOutOfRangeException ex)
            {
                MessageBox.Show("Argument nicht innerhalb der erlaubten Werte:\n" + ex.Message);
            }
        }

        private void optimization_b_Click(object sender, EventArgs e)
        {
            if (null == Program.rawData)
            {
                MessageBox.Show("Please load a data set.", "No Data Set",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            if (null == Program.CpAlgo)
            {
                MessageBox.Show("Please select a change detection algorithm.", "No Change Detection Algorithm",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            int[] changePoints = null;
            try
            {
                changePoints = Program.CpAlgo.GetChangePoints();
            }
            catch (InvalidOperationException)
            {

            }
            if (null == changePoints)
            {
                MessageBox.Show("Please run change detection algorithm to create initial set of change points.", "No Change Points",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            if (null == Program.CompAlgo)
            {
                MessageBox.Show("Please select a compression algorithm.", "No Compression Algorithm",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            if (null == Program.OptAlgo)
            {
                MessageBox.Show("Please select an optimization algorithm.", "No Optimization Algorithm",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            Optimizer optimizer = new Optimizer(Program.rawData, Program.CpAlgo, Program.CompAlgo, Program.OptAlgo);
            try
            {
                optimizer.SetParameters(Utilities.ListToDictionary((List<Parameter>)fitnessParametersGrid.DataSource, optimizer.GetParameters()));
            }
            catch (ArgumentOutOfRangeException ex)
            {
                MessageBox.Show("Argument nicht innerhalb der erlaubten Werte:\n" + ex.Message);
            }

            // starting optimization in new thread //
            status_tsl.Text = "Running optimization";
            showProgress(true);
            optimizationWorker.RunWorkerAsync(optimizer);
        }

        private void optimizationWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = ((Optimizer)e.Argument).createResult(((Optimizer)e.Argument).Solve());

        }

        private void optimizationWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {

            DeutscheBahn.JobResult jr = (DeutscheBahn.JobResult)e.Result;
            MessageBox.Show("ChangeDetection:" + jr.ChangeDetectionParameters + "\n" + "Compression:" + jr.CompressionParameters + "\n"
                + "Fitness:" + jr.Fitness + "\nRMSE:" + jr.RMSE + "\nCR:" + jr.CompressionRatio);
            showProgress(false);
        }

        private void compressWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (Program.CompAlgo != null)
            {
                graphsForm.SetCompressionStats(Program.CompAlgo.getCompressionRatio(),
                    Utilities.calculateRMSE(Program.rawData, Program.decompData));
                graphsForm.SetDecompressedData(Program.decompData);
            }

            showProgress(false);
        }

        private void compressWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            if (Program.CompAlgo != null)
            {
                Program.CompAlgo.SetDataset(Program.rawData);
                Program.CompAlgo.CompressDataset();
                Program.decompData = Program.CompAlgo.GetDecompressedDataset();

            }
        }

        private void measureWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                MeasureResult measure = ChangeDetectionMass.get().calculateMeasure(Program.rawData, Program.rawChangePoints,
                    null, Program.decompData, Program.decompChangePoints, null);
                e.Result = measure;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error:" + ex,
                 "Fehler beim Maß",
                 MessageBoxButtons.OK,
                 MessageBoxIcon.Exclamation,
                 MessageBoxDefaultButton.Button1);
            }
        }

        private void measureWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            showProgress(false);

            MeasureResult measure = (MeasureResult)e.Result;

            if (null == measure) return;

            Optimizer o = new Optimizer(new float[0], new float[0], new int[0], null, null, null);
            float CR = 1;//Set Optimizer per defaut to 1 for that it cannot crash if not compressed
            if (Program.CompAlgo != null)
                CR = Program.CompAlgo.getCompressionRatio();
            o.SetParameters(Utilities.ListToDictionary((List<Parameter>)fitnessParametersGrid.DataSource, o.GetParameters()));
            double f = o.getFitness(o.getTotalError(measure), CR);
            graphsForm.SetMeasureStats(measure, (float)f);
        }

        private void cpWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            // CP for raw data //
            Program.CpAlgo.SetDataset(Program.rawData);
            Program.CpAlgo.DetectChanges();

            Program.rawChangePointScores = Program.CpAlgo.GetScores();
            Program.rawChangePoints = Program.CpAlgo.GetChangePoints();

            if (Program.decompData == null)
            {
                return;
            }

            // CP for compressed data //
            Program.CpAlgo.SetDataset(Program.decompData);
            Program.CpAlgo.DetectChanges();

            Program.decompChangePointScores = Program.CpAlgo.GetScores();
            Program.decompChangePoints = Program.CpAlgo.GetChangePoints();
        }

        private void cpWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            graphsForm.SetRawChangePoints(Program.rawChangePoints);

            if (Program.decompData == null)
            {
                showProgress(false);
                return;
            }

            graphsForm.SetDecompressedChangePoints(Program.decompChangePoints);

            status_tsl.Text = "Calculating change point measure";
            measureWorker.RunWorkerAsync();
        }

        private void toolStripCancelLabel_Click(object sender, EventArgs e)
        {
            if (compressWorker.IsBusy) compressWorker.CancelAsync();
            if (measureWorker.IsBusy) measureWorker.CancelAsync();
            if (cpWorker.IsBusy) cpWorker.CancelAsync();

            showProgress(false);
        }

        private void showProgress(bool show)
        {
            progress_tsb.Visible = show;
            status_tsl.Visible = show;
        }

        private void showLoading(bool show)
        {
            showLoading(show, "Please wait. Loading...");
        }

        private void showLoading(bool show, string text)
        {
            if (loadingForm == null) loadingForm = new GUI_loading();
            loadingForm.Visible = show;
            loadingForm.setText(text);
            loadingForm.Update();
        }

        private void uploadData_b_Click(object sender, EventArgs e)
        {
            // load data //
            openForm.Title = "Choose Dataset";
            openForm.ShowDialog(this);
            if (openForm.FileName == "") return;

            showLoading(true);
            CsvImporter importer = new CsvImporter(openForm.FileName);
            importer.ReadFile();
            float[] rawData = importer.GetColumn(0);

            // upload data //
            using (var context = new DeutscheBahn.Model1Container())
            {
                List<DeutscheBahn.Data> datalist = new List<DeutscheBahn.Data>();
                for (int i = 0; i < rawData.Length; i++)
                {
                    float raw = rawData[i];
                    DeutscheBahn.Data dobj = new DeutscheBahn.Data();
                    dobj.time = i;
                    dobj.isChange = false;
                    dobj.value = raw;
                    datalist.Add(dobj);
                }
                DeutscheBahn.DataSet ds = new DeutscheBahn.DataSet();
                ds.data = datalist;
                ds.name = System.IO.Path.GetFileName(openForm.FileName);

                context.DataSetSet1.Add(ds);
                context.SaveChanges();
            }
            showLoading(false);
            // refresh //
            this.dataSetSet1TableAdapter.Fill(this.pdfDataSet.DataSetSet1);
        }

        private void scriptWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            startTimer.Enabled = false;
            using (var database = new DeutscheBahn.Model1Container())
            {

                DeutscheBahn.Job job = null;
                try
                {
                    using (var transaction = database.Database.BeginTransaction())
                    {
                        try
                        {
                            var query = (from jobs in database.JobSet where jobs.status == Job.stat.Open.ToString() select jobs).Take(1);
                            List<DeutscheBahn.Job> jobset = query.ToList<DeutscheBahn.Job>();
                            if (jobset.Count > 0)
                            {
                                jobset[0].JobStatus = DeutscheBahn.Job.stat.InProgress;
                                jobset[0].starttime = DateTime.Now;
                                jobset[0].resolver = Environment.MachineName;
                                job = jobset[0];
                                database.SaveChanges();
                            }

                            transaction.Commit();
                        }
                        catch (Exception)
                        {
                            transaction.Rollback();
                        }
                    }
                }
                catch (System.Data.Entity.Core.EntityException)
                {
                    //TODO Error Handling
                }

                if (null != job)
                {
                    try
                    {
                        Script script = new Script(job);
                        string label = script.JobName + "\n";
                        label += "using " + script.CompAlgoName + ", " + script.CPAlgoName + " and " + script.OptAlgoName + "\n";
                        label += "on dataset " + script.DatasetName + "\n";
                        scripting_label.Invoke((MethodInvoker)delegate
                        {
                            scripting_label.Text = label;
                            status_tsl.Text = script.JobName;
                            showProgress(true);
                        });

                        script.Start();
                    }
                    catch (Exception error)
                    {
                        if (error is InvalidOperationException) throw; // that means the DB has an error
                        if (!(error is System.Data.Entity.Core.EntityException))
                        {
                            job.JobStatus = DeutscheBahn.Job.stat.Failed;
                            //Exception has in first line the lastFitnessParams
                            job.exception = Program.lastFitnessParams + "\n";
                            job.exception += error.Message;
                            job.stacktrace = error.StackTrace;
                            job.endtime = DateTime.Now;
                            database.SaveChanges();
                        }
                    }
                }
            }
        }

        private void scriptWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            showProgress(false);
            autostart_cb(null, null);
        }

        private void autostart_cb(object sender, EventArgs e)
        {
            if (!scriptWorker.IsBusy && autoStart_cb.Checked)
            {
                startTimer.Enabled = true;
                scripting_label.Text = "Looking for open jobs...";
            }
            else
            {
                startTimer.Enabled = false;
                scripting_label.Text = "Auto start not enabled.";
            }
        }

        private void startTimer_Tick(object sender, EventArgs e)
        {
            scriptWorker.RunWorkerAsync();
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            GUI_about aboutForm = new GUI_about();
            aboutForm.Show();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void GUI_main_Load(object sender, EventArgs e)
        {
            try
            {
                // TODO: This line of code loads data into the 'pdfDataSet.JobSet' table. You can move, or remove it, as needed.
                this.jobSetTableAdapter.Fill(this.pdfDataSet.JobSet);
                // TODO: This line of code loads data into the 'pdfDataSet.DataSetSet1' table. You can move, or remove it, as needed.
                this.dataSetSet1TableAdapter.Fill(this.pdfDataSet.DataSetSet1);
            }
            catch (MySql.Data.MySqlClient.MySqlException error)
            {
                //TODO error handling??
            }
        }

        private void createJob()
        {
            if ((task.CompressionAlgorithm == null) || (task.CPAlgorithm == null) || (task.OptimizationAlgorithm == null))
            {
                MessageBox.Show("Please choose a compression, change detection and optimization technique.", "No algorithm was chosen",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // add dataset id to task  //
            task.Dataset = data_g.SelectedCells[0].Value.ToString();

            // add optimize and fitness parameter to task //
            Optimizer tempFitness = new Optimizer(new float[0], new float[0], new int[0], null, null, null);
            task.OptFitnessParameters = Utilities.DictRVToDictFloat(Utilities.ListToDictionary((List<Parameter>)fitnessParametersGrid.DataSource, tempFitness.GetParameters()));
            task.OptimizationParameters = Utilities.DictRVToDictFloat(Utilities.ListToDictionary((List<Parameter>)optParametersGrid.DataSource, Program.OptAlgo.GetParameters()));

            // add comp parameter to task, only if it is not the search for find initial //
            if (task.CompressionAlgorithm != initial_rb.Text)
                task.CompressionInitialParameters = Utilities.DictRVToDictFloat(Utilities.ListToDictionary((List<Parameter>)compParametersGrid.DataSource, Program.CompAlgo.GetParameters()));
            // add cp parameter to task //
            task.CPInitialParameters = Utilities.DictRVToDictFloat(Utilities.ListToDictionary((List<Parameter>)cpParametersGrid.DataSource, Program.CpAlgo.GetParameters()));

            // push task to db //
            List<Task> taskList = new List<Task>();
            taskList.Add(task);
            using (var db = new DeutscheBahn.Model1Container())
            {
                DeutscheBahn.Job job = new DeutscheBahn.Job();
                job.json = JsonConvert.SerializeObject(taskList);
                job.JobStatus = Job.stat.Open;
                job.name = name_box.Text;
                int dsid = int.Parse(task.Dataset);
                var query = from ds in db.DataSetSet1 where ds.Id == dsid select ds;
                job.dataset = query.ToArray()[0];
                db.JobSet.Add(job);
                try
                {
                    db.SaveChanges();
                }
                catch (System.Data.Entity.Validation.DbEntityValidationException dbEx)
                {
                    Exception raise = dbEx;
                    foreach (var validationErrors in dbEx.EntityValidationErrors)
                    {
                        foreach (var validationError in validationErrors.ValidationErrors)
                        {
                            string message = string.Format("{0}:{1}",
                                validationErrors.Entry.Entity.ToString(),
                                validationError.ErrorMessage);
                            // raise a new exception nesting the current instance as InnerException
                            raise = new InvalidOperationException(message, raise);
                        }
                    }
                    throw raise;
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message, "Failed to create job!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            // refresh job list //
            this.jobSetTableAdapter.Fill(this.pdfDataSet.JobSet);
        }

        private void newJob_b_Click(object sender, EventArgs e)
        {
            createJob();
        }

        private void opt_rbs_CheckedChanged(object sender, EventArgs e)
        {
            if (evo_rb.Checked)
            {
                Program.OptAlgo = new EvolutionaryOptimization();
                task.OptimizationAlgorithm = "Evo";
            }
            if (nelderMead_rb.Checked)
            {
                Program.OptAlgo = new NelderMeadSolver();
                task.OptimizationAlgorithm = "NM";
            }
            if (nsga2_rb.Checked)
            {
                Program.OptAlgo = new NSGA2();
                task.OptimizationAlgorithm = "NSGA2";
            }

            if (fitnessParametersGrid.DataSource == null)
            {
                Optimizer tempFitness = new Optimizer(new float[0], new float[0], new int[0], null, null, null);
                fitnessParametersGrid.DataSource = Utilities.DictionaryToList(tempFitness.GetParameters());
                fitnessParametersGrid.Columns[0].ReadOnly = true;
                fitnessParametersGrid.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                fitnessParametersGrid.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            }

            optParametersGrid.DataSource = Utilities.DictionaryToList(Program.OptAlgo.GetParameters());
            optParametersGrid.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            optParametersGrid.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        }

        private void loadDB_tsb_Click(object sender, EventArgs e)
        {
            // reset data objects and GUI//
            Program.rawData = null;
            Program.decompData = null;
            Program.rawChangePointScores = null;
            Program.decompChangePointScores = null;
            Program.rawChangePoints = null;
            Program.decompChangePoints = null;
            graphsForm.ClearGraphs();

            // get data from db //
            if (data_g.SelectedRows.Count != 0)
            {
                int datasetID = (int)data_g.SelectedCells[0].Value;
                using (var database = new DeutscheBahn.Model1Container())
                {
                    var query = (from dataset in database.DataSetSet1 where dataset.Id == datasetID select dataset).Take(1);
                    Program.rawData = query.ToList<DeutscheBahn.DataSet>()[0].DataArray;
                    Program.taggedData = query.ToList<DeutscheBahn.DataSet>()[0];
                    Program.rawChangePoints = Program.taggedData.expectedCP;
                    Program.rawChangePointScores = Program.taggedData.expectedScores;
                    //MessageBox.Show(data_g.SelectedCells[1].Value + " loaded successfully.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            // put data in graphs //
            graphsForm.SeriesName = (string)data_g.SelectedCells[1].Value;
            graphsForm.SetRawData(Program.rawData);
            graphsForm.SetRawChangePoints(Program.taggedData.expectedCP);

            // show graphs window //
            graphsForm.Show();
            graphsForm.BringToFront();
        }

        private void graphsAndResultsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            graphsForm.Show();
            graphsForm.BringToFront();
        }

        private void findInitial_Click(object sender, EventArgs e)
        {
            showLoading(true);
            if (Program.CpAlgo == null)
            {
                MessageBox.Show("No ChangeDetection Algorithm chosen!");
                showLoading(false);
                return;
            }
            if (Program.taggedData == null || Program.taggedData.DataArray.Length == 0)
            {
                MessageBox.Show("No data was chosen or chosen data is empty.", "No Data", MessageBoxButtons.OK, MessageBoxIcon.Error);
                showLoading(false);
                return;
            }
            //Here the ChangeDetection Optimization is trained! under the assumption that we find shit ;)
            IOptimizationAlgorithm optAlgo = new EvolutionaryOptimization();
            Program.CpAlgo.SetDataset(Program.rawData);
            Program.CpAlgo.DetectChanges();
            System.Diagnostics.Debug.WriteLine("Creating Optimizer");
            Optimizer optimizer = new Optimizer(Program.taggedData.DataArray, Program.taggedData.expectedScores, Program.taggedData.expectedCP, Program.CpAlgo, optAlgo); ;
            optimizer.SetOptimizationParameters(2, 1);
            optimizer.SetMeasurementParameters(1, 1, 1);
            //Then the optimization is going on! what a crazy party ;)
            RangedValue[] bestParams = optimizer.Solve();
            Dictionary<String, RangedValue> parametersDict = Program.CpAlgo.GetParameters();
            String output = "";
            for (int i = 0; i < bestParams.Length; i++)
            {
                output += parametersDict.ElementAt(i).Key + ": " + bestParams[i].Value + "\n";
            }

            DeutscheBahn.JobResult r = optimizer.createResult(bestParams);
            output = "Fitness: " + r.Fitness + "\npc: " + r.PC + "\nmiss: " + r.MISS + "\nfp: " + r.FP + "\n" + output;
            Console.WriteLine(output);
            showLoading(false);
            MessageBox.Show(output, "Initial Parameterset was found", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.jobSetTableAdapter.Fill(this.pdfDataSet.JobSet);
            this.dataSetSet1TableAdapter.Fill(this.pdfDataSet.DataSetSet1);
        }

        private void data_g_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            loadDB_tsb_Click(null, null);
        }

        private void setInitialParameters()
        {
            showLoading(true);
            try
            {
                if ((Program.CpAlgo == null) || (data_g.SelectedCells == null) || (data_g.SelectedCells.Count == 0))
                {
                    showLoading(false);
                    return;
                }

                int datasetID = (int)data_g.SelectedCells[0].Value;
                string cdparams = "";
                using (var database = new DeutscheBahn.Model1Container())
                {
                    var query = (from jobset in database.JobSet
                                 join resultSet in database.JobResultSet on jobset.Id equals resultSet.Job.Id
                                 where jobset.json.Contains("CompressionAlgorithm\":\"Find Initial")
                                 && jobset.json.Contains("CPAlgorithm\":\"" + Program.CpAlgo.Name)
                                 && jobset.json.Contains("Dataset\":\"" + datasetID)
                                 && resultSet.isGoodFinalResult == true
                                 select jobset.Id).Take(1);

                    if (query.Count() == 0)
                    {
                        showLoading(false);
                        return;
                    }
                    int jobResultID = query.ToList<int>()[0];
                    var paraQuery = (from jobResults in database.JobResultSet where jobResults.Job.Id == jobResultID select jobResults.ChangeDetectionParameters);

                    cdparams = paraQuery.ToList<string>()[0];
                }
                string[] t = cdparams.Split(new[] { ';' }, StringSplitOptions.RemoveEmptyEntries);

                Dictionary<string, RangedValue> algoparams = Program.CpAlgo.GetParameters();
                foreach (string pair in t)
                {
                    string key = pair.Split('=')[0];
                    float value = float.Parse(pair.Split('=')[1], CultureInfo.InvariantCulture);
                    if (algoparams.ContainsKey(key))
                        algoparams[key].Value = value;
                }


                if ((algoparams != null) && (algoparams.Count() != 0))
                {
                    cpParametersGrid.DataSource = Utilities.DictionaryToList(algoparams);
                    Program.CpAlgo.SetParameters(algoparams);
                }
            }
            catch (Exception error)
            {
                showLoading(false);
                MessageBox.Show("There occured an error retrieving the initial cd parameters from db.\nThe appclication is still running.\n\n" + error.Message, "No Initials", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            showLoading(false);
        }

        private void data_g_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            setInitialParameters();
        }

        private void data_g_Click(object sender, EventArgs e)
        {
            setInitialParameters();
        }

        private void exportCsvButton_Click(object sender, EventArgs e)
        {
            saveFileDialog.ShowDialog(this);
            CsvExporter.export(saveFileDialog.FileName, Program.decompData, Program.decompChangePoints);
        }

        private void findInitial_Click_1(object sender, EventArgs e)
        {
            //Check if Raw Data (from DB) is available
            if (Program.rawData == null || Program.rawChangePoints == null || Program.rawChangePointScores == null)
            {
                MessageBox.Show("Please load annotated data from database first");
                return;
            }
            //Check if Changedetection is chosen.
            if (Program.CpAlgo == null)
            {
                MessageBox.Show("Please choose a change detection algorithm!", "No Change Detection Algorithm",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            Program.decompData = Program.rawData;

            //Run ChangeDetection on RawData
            Program.CpAlgo.SetDataset(Program.rawData);
            Program.CpAlgo.DetectChanges();
            //But sotre in Decomp since this is the bottom Graph
            Program.decompChangePointScores = Program.CpAlgo.GetScores();
            Program.decompChangePoints = Program.CpAlgo.GetChangePoints();

            graphsForm.SetRawChangePoints(Program.rawChangePoints);

            graphsForm.SetDecompressedChangePoints(Program.decompChangePoints);
            graphsForm.SetDecompressedData(Program.rawData);


            measureWorker.RunWorkerAsync();
        }

        private void general_tsb_Click(object sender, EventArgs e)
        {
            showLoading(true);

            // convert dataset ID //
            int datasetID = (int)data_g.SelectedCells[0].Value;
            switch (datasetID)
            {
                case 18: datasetID = 38; break;
                case 30: datasetID = 20; break;
                case 13: datasetID = 32; break;
                case 24: datasetID = 26; break;
                case 37: datasetID = 36; break;
                default: MessageBox.Show("no generalization possible on dataset " + datasetID, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                    
            }

            // get CD initial parameters //
            showLoading(true);
            try
            {
                if ((Program.CpAlgo == null) || (data_g.SelectedCells == null) || (data_g.SelectedCells.Count == 0))
                {
                    showLoading(false);
                    MessageBox.Show("Please choose comp and CD algo and dataset", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                string cdparams = "";
                using (var database = new DeutscheBahn.Model1Container())
                {
                    var query = (from jobset in database.JobSet
                                 join resultSet in database.JobResultSet on jobset.Id equals resultSet.Job.Id
                                 where jobset.json.Contains("CompressionAlgorithm\":\"Find Initial")
                                 && jobset.json.Contains("CPAlgorithm\":\"" + Program.CpAlgo.Name)
                                 && jobset.json.Contains("Dataset\":\"" + datasetID)
                                 && resultSet.isGoodFinalResult == true
                                 select jobset.Id).Take(1);

                    if (query.Count() == 0)
                    {
                        showLoading(false);
                        return;
                    }
                    int jobResultID = query.ToList<int>()[0];
                    var paraQuery = (from jobResults in database.JobResultSet where jobResults.Job.Id == jobResultID select jobResults.ChangeDetectionParameters);

                    cdparams = paraQuery.ToList<string>()[0];
                }
                string[] t = cdparams.Split(new[] { ';' }, StringSplitOptions.RemoveEmptyEntries);

                Dictionary<string, RangedValue> algoparams = Program.CpAlgo.GetParameters();
                foreach (string pair in t)
                {
                    string key = pair.Split('=')[0];
                    float value = float.Parse(pair.Split('=')[1], CultureInfo.InvariantCulture);
                    if (algoparams.ContainsKey(key))
                        algoparams[key].Value = value;
                }


                if ((algoparams != null) && (algoparams.Count() != 0))
                {
                    //cpParametersGrid.DataSource = Utilities.DictionaryToList(algoparams);
                    Program.CpAlgo.SetParameters(algoparams);
                }
            }
            catch (Exception error)
            {
                showLoading(false);
                MessageBox.Show("There occured an error retrieving the initial cd parameters from db.\nThe appclication is still running.\n\n" + error.Message, "No Initials", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            // CD on raw data //
            Program.CpAlgo.SetDataset(Program.rawData);
            Program.CpAlgo.DetectChanges();
            Program.rawChangePointScores = Program.CpAlgo.GetScores();
            Program.rawChangePoints = Program.CpAlgo.GetChangePoints();

            // show change points on raw data//
            graphsForm.SetRawChangePoints(Program.rawChangePoints);

            // push parameters from GUI to Programm.cs //
            Program.CompAlgo.SetParameters(Utilities.ListToDictionary((List<Parameter>)compParametersGrid.DataSource, Program.CompAlgo.GetParameters()));
            Program.CpAlgo.SetParameters(Utilities.ListToDictionary((List<Parameter>)cpParametersGrid.DataSource, Program.CpAlgo.GetParameters()));
       
            // do compression //
            status_tsl.Text = "Compressing data";
            showProgress(true);
            Program.CompAlgo.SetDataset(Program.rawData);
            Program.CompAlgo.CompressDataset();
            Program.decompData = Program.CompAlgo.GetDecompressedDataset();
            graphsForm.SetCompressionStats(Program.CompAlgo.getCompressionRatio(), Utilities.calculateRMSE(Program.rawData, Program.decompData));
            graphsForm.SetDecompressedData(Program.decompData);

            // CD on comp data //
            Program.CpAlgo.SetDataset(Program.decompData);
            Program.CpAlgo.DetectChanges();
            Program.decompChangePointScores = Program.CpAlgo.GetScores();
            Program.decompChangePoints = Program.CpAlgo.GetChangePoints();

            // show change points on compressed data //
            graphsForm.SetDecompressedChangePoints(Program.decompChangePoints);

            // do MILTON //
            status_tsl.Text = "Calculating change point measure";
            measureWorker.RunWorkerAsync();

            showLoading(false);
        }
    }
}
