﻿using System;

namespace Framework
{
    public class RangedValue
    {
        private bool isInt = false;
        public bool IsInt
        {
            get { return isInt; }
            set { isInt = value; }
        }

        /// <summary>
        /// Actual Value. Throws ArgumentOutOfRangeException if Value is not between Min and Max.
        /// </summary>
        private float val;
        public float Value
        {
            get { return val; }
            set
            {
                float tmp = isInt ? (int)value : value;
                if (tmp.CompareTo(minimum) < 0 || tmp.CompareTo(maximum) > 0)
                {
                    throw new ArgumentOutOfRangeException("Value must be between " + minimum + " and " + maximum + ".");
                }
                val = tmp;
            }
        }

        /// <summary>
        /// Minimum Value
        /// </summary>
        private float minimum;
        public float Min
        {
            get { return minimum; }
            set { minimum = value; }
        }

        /// <summary>
        /// Max Value
        /// </summary>
        private float maximum;
        public float Max
        {
            get { return maximum; }
            set { maximum = value; }
        }

        /// <summary>
        /// Set All values on one time.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="min"></param>
        /// <param name="max"></param>
        public RangedValue(float value, float min, float max, bool isInt)
        {
            if (value < min || value > max)
                throw new ArgumentOutOfRangeException("Value must be between " + min + " and " + max + ".");
            this.val = value;
            this.minimum = min;
            this.maximum = max;
            this.isInt = isInt;
        }

        public virtual RangedValue Clone()
        {
            //return (RangedValue)this.MemberwiseClone();
            return (RangedValue)Convert.ChangeType(this.MemberwiseClone(), this.GetType());
        }

        public static RangedValue[] CopyArray(RangedValue[] array)
        {
            RangedValue[] result = new RangedValue[array.Length];
            for (int i = 0; i < array.Length; i++)
            {
                result[i] = array[i].Clone();
            }
            return result;
        }
    }

    public class RangedIntValue : RangedValue
    {
        public RangedIntValue(int value, int min, int max)
            : base(value, min, max, true)
        {

        }

        public override RangedValue Clone()
        {
            return (RangedIntValue)this.MemberwiseClone();
        }
    }

    public class RangedFloatValue : RangedValue
    {
        public RangedFloatValue(float value, float min, float max)
            : base(value, min, max, false)
        {

        }

        public override RangedValue Clone()
        {
            return (RangedFloatValue)this.MemberwiseClone();
        }
    }
}
