﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Framework
{
    class Utilities
    {
        public static float calculateAbsoluteEpsilon(float[] rawData, float relativeEpsilon)
        {
            if (null == rawData || 0 == rawData.Length)
            {
                throw new ArgumentNullException("rawData must contain data.");
            }

            float min = rawData[0];
            float max = rawData[0];

            for (int i = 0; i < rawData.Length; ++i)
            {
                if (rawData[i] < min)
                {
                    min = rawData[i];
                }
                if (rawData[i] > max)
                {
                    max = rawData[i];
                }
            }

            float absoluteEpsilon = (max - min) * relativeEpsilon;
            return absoluteEpsilon;
        }

        public static float calculateRMSE(float[] rawData, float[] decompData)
        {
            int length = (decompData.Length > rawData.Length) ? rawData.Length : decompData.Length;
            double rmse = 0;

            for (int i = 0; i < length; i++)
            {
                rmse += Math.Pow((decompData[i] - rawData[i]), 2);
            }
            return (float)Math.Pow((rmse / length), 0.5);
        }

        public static List<Parameter> DictionaryToList(Dictionary<string, RangedValue> dict)
        {
            List<Parameter> list = new List<Parameter>();
            foreach (KeyValuePair<string, RangedValue> item in dict)
            {
                list.Add(new Parameter(item.Key, (float)item.Value.Value));
            }
            return list;
        }
        public static Dictionary<string, float> DictRVToDictFloat(Dictionary<string, RangedValue> dict)
        {
            Dictionary<string, float>  result = new Dictionary<string,float>();
            foreach (KeyValuePair<string, RangedValue> item in dict)
            {
                result.Add(item.Key, item.Value.Value);
            }
            return result;
        }

        public static Dictionary<string, RangedValue> ListToDictionary(List<Parameter> list, Dictionary<string, RangedValue> old)
        {
            Dictionary<string, RangedValue> dict = new Dictionary<string, RangedValue>();
            foreach (Parameter item in list)
            {
                RangedValue oldvalue = (RangedValue)old[item.Key];
                oldvalue.Value = item.Value;
                dict.Add(item.Key, oldvalue);
            }
            return dict;
        }

        public static string DictToString(Dictionary<string, RangedValue> dict)
        {
            string result = "";
            foreach (KeyValuePair<String, RangedValue> item in dict)
            {
                result = result + item.Key + ": " + item.Value.Value + "\n";
            }
            return result;
        }
    }

    class Parameter
    {
        public Parameter(String s, float f)
        {
            Key = s;
            Value = f;
        }
        public string Key { get; set; }
        public float Value { get; set; }
    }
}
