﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ADWINLIB;

namespace Framework.changedetection
{
    class ADWIN : IChangeDetectionAlgorithm
    {
        RangedIntValue M = new RangedIntValue(5, 2, 10);
        const string M_K = "M";
        RangedFloatValue delta = new RangedFloatValue(0.1f, 0, 1);
        const string delta_K = "delta";

        private float[] values;
        private int[] changes;
        private bool hasRun = false;

        public void SetParameters(Dictionary<string, RangedValue> parameters)
        {
            foreach (KeyValuePair<string, RangedValue> kvp in parameters)
            {
                switch (kvp.Key)
                {
                    case M_K:
                        RangedIntValue value = (RangedIntValue)kvp.Value;
                        if ((int)value.Value > 0)
                        { //Only take if Value is >0
                            M.Value = value.Value;
                        }
                        break;
                    case delta_K:
                        delta.Value = ((RangedFloatValue)kvp.Value).Value;
                        break;
                    default:
                        throw new InvalidOperationException("Parameter " + kvp.Key + " existiert in Algorithmus nicht");
                }
            }
        }

        public Dictionary<string, RangedValue> GetParameters()
        {
            Dictionary<string, RangedValue> parameters = new Dictionary<string, RangedValue>();
            parameters.Add(M_K, M);
            parameters.Add(delta_K, delta);
            return parameters;
        }

        public void SetDataset(float[] data)
        {
            this.values = data;
        }

        public void DetectChanges()
        {
            AdwinAlgo adl = new AdwinAlgo();
            changes = adl.runAdwin(values, (int)M.Value, delta.Value);
            hasRun = true;
        }

        public bool HasScores()
        {
            return false;
        }

        public float[] GetScores()
        {
            if (!hasRun)
            {
                throw new InvalidOperationException("Change detection was not executed. Call DetectChanges() first.");
            }
            float[] scores = new float[values.Length];
            for (int i = 0; i < changes.Length; i++) {
                scores[changes[i]] = 1;
            }
            return scores;
        }

        public int[] GetChangePoints()
        {
            if (!hasRun)
            {
                throw new InvalidOperationException("Change detection was not executed. Call DetectChanges() first.");
            }

            return changes;
        }


        public Dictionary<string, RangedValue> GetOptimizationParameters()
        {
            return GetParameters();
        }

        public string Name
        {
            get { return "ADWIN"; }
        }
    }
}
