﻿using MathWorks.MATLAB.NET.Arrays;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Framework.changedetection
{
    class ChangeFinderCD : IChangeDetectionAlgorithm
    {
        RangedIntValue T = new RangedIntValue(5, 3, 10);
        RangedIntValue k = new RangedIntValue(3, 2, 10);
        RangedIntValue log = new RangedIntValue(0, 0, 1);
        RangedFloatValue r = new RangedFloatValue(0.1f, 0f, 0.4f);
        RangedIntValue threshold = new RangedIntValue(2, 0, 5000);

        private float[] data;
        private float[] scores;
        private int[] changePoints;
        private bool hasScores = true;
        private bool hasRun = false;


        public void SetParameters(Dictionary<string, RangedValue> parameters)
        {
            if (parameters.ContainsKey("T"))
                this.T = (RangedIntValue)parameters["T"];
            if (parameters.ContainsKey("k"))
                this.k = (RangedIntValue)parameters["k"];
            //if (parameters.ContainsKey("log"))
            //    this.log = (RangedIntValue)parameters["log"];
            if (parameters.ContainsKey("r"))
                this.r = (RangedFloatValue)parameters["r"];
            if (parameters.ContainsKey("threshold"))
                this.threshold = (RangedIntValue)parameters["threshold"];
        }

        public Dictionary<string, RangedValue> GetParameters()
        {
            Dictionary<string, RangedValue> parameters = new Dictionary<string, RangedValue>();
            parameters.Add("T", T);
            parameters.Add("k", k);
            //parameters.Add("log", log);
            parameters.Add("r", r);
            parameters.Add("threshold", threshold);
            return parameters;
        }

        public Dictionary<string, RangedValue> GetOptimizationParameters()
        {
            return GetParameters();
        }

        public void SetDataset(float[] data)
        {
            this.data = data;
        }

        public void DetectChanges()
        {
            MWArray[] result;
            try
            {
                ChangeFinder.ChangeFinderClass detector = new ChangeFinder.ChangeFinderClass();
                result = detector.ChangeFinder(1,
                                              new MWNumericArray(this.data),
                                              new MWNumericArray((float)T.Value),
                                              new MWNumericArray((float)k.Value),
                                              new MWNumericArray((float)r.Value),
                                              new MWNumericArray((float)log.Value));
            }
            catch (TypeInitializationException e)
            {
                throw new TypeInitializationException("Could not initialize Matlab runtime.", e.InnerException);
            }



            scores = new float[result[0].NumberOfElements];
            int i = 0;
            foreach (double score in result[0].ToArray())
            {
                scores[i] = (float)score;
                i++;
            }
            //scores = (float[]) result[0].ToArray();


            List<int> tmpCPs = new List<int>();

            for (int j = 0; j < scores.Length; j++)
            {
                if (scores[j] > threshold.Value)
                {
                    tmpCPs.Add(j);
                }
            }

            changePoints = tmpCPs.ToArray();
            hasRun = true;
        }

        public bool HasScores()
        {
            return this.hasScores;
        }

        public float[] GetScores()
        {
            if (!hasRun)
            {
                throw new InvalidOperationException("Change detection was not executed. Call DetectChanges() first.");
            }

            return this.scores;
        }

        public int[] GetChangePoints()
        {
            if (!hasRun)
            {
                throw new InvalidOperationException("Change detection was not executed. Call DetectChanges() first.");
            }

            return this.changePoints;
        }

        public string Name
        {
            get { return "ChangeFinder"; }
        }
    }
}
