﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Framework.changedetection
{
    interface IChangeDetectionAlgorithm
    {
        string Name {get;}
        //string Name { get; }
        /// <summary>
        /// Sets the parameters for the algorithm. Since each algorithm has its own parameters a string-float dictionary is used.
        /// </summary>
        /// <param name="parameters">A dictionary of parameter names and values.</param>
        void SetParameters(Dictionary<string, RangedValue> parameters);

        /// <summary>
        /// Gets the parameters for the algorithm. Since each algorithm has its own parameters a string-float dictionary is used.
        /// </summary>
        Dictionary<string, RangedValue> GetParameters();

        /// <summary>
        /// Returns the parameter set which are relevant for the optimization. 
        /// This is a proper subset of GetParameters
        /// </summary>
        Dictionary<String, RangedValue> GetOptimizationParameters();

        /// <summary>
        /// Sets the data to perform change detection on.
        /// </summary>
        /// <param name="data">An array of data values</param>
        void SetDataset(float[] data);

        /// <summary>
        /// Run the change detecton algorithm.
        /// </summary>
        void DetectChanges();

        /// <summary>
        /// Not all change detection algorithms calculate a score for each time slot.
        /// </summary>
        /// <returns>Ture, when algorithm calculates scores.</returns>
        bool HasScores();

        /// <summary>
        /// Returns the calculated scores after detectChanges was executed. If called before detectChanges 
        /// was executed it will throw an exception. If the algorithm does not provide scores, this returns null.
        /// </summary>
        /// <returns>Returns the calculated scores or null if the algoorithm does not provide scores.</returns>
        float[] GetScores();

        /// <summary>
        /// Returns the calulated change points after detectChanges was executed. If called before detectChanges 
        /// was executed it will throw an exception.
        /// </summary>
        /// <returns>Returns the calculated change points.</returns>
        int[] GetChangePoints();
    }
}
