﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Framework.compression.algorithms
{
    class CHEB : ICompressionAlgorithm
    {     
        //parameters and there Key Strings
        RangedFloatValue epsilon = new RangedFloatValue(0.05f, 0, 0.3f); const string EPSILON_K = "epsilon";
        RangedIntValue segmentlength = new RangedIntValue(4, 4, int.MaxValue); const string SEGLENGTH_K = "segmentlength";

        //Dataset to Compress
        float[] values;
        //Compressed Dataset
        float[] decompressed;
        float compressionration;

        //is true if compression has been done with current parameters and values, else false.
        bool compressionDone = false;

        public CHEB()
        {
        }

        public void SetParameters(Dictionary<string, RangedValue> parameters)
        {
            foreach (KeyValuePair<string, RangedValue> kvp in parameters)
            {
                switch (kvp.Key)
                {
                    case EPSILON_K:
                        epsilon.Value = ((RangedFloatValue)kvp.Value).Value;
                        compressionDone = false;
                        break;
                    case SEGLENGTH_K:
                        segmentlength.Value = ((RangedIntValue)kvp.Value).Value;
                        compressionDone = false;
                        break;
                    default:
                        throw new InvalidOperationException("Prameter " + kvp.Key + " existiert in Algorithmus nicht");
                }
            }
        }

        public Dictionary<string, RangedValue> GetParameters()
        {
            Dictionary<string, RangedValue> parameters = new Dictionary<string, RangedValue>();
            parameters.Add(EPSILON_K, epsilon);
            parameters.Add(SEGLENGTH_K, segmentlength);
            return parameters;
        }

        public void SetDataset(float[] data)
        {
            //If less than 2 Entries there is nothing to compress and caller seems to be awkward ...
            if (data.Length < 2)
            {
                throw new ArgumentOutOfRangeException();
            }

            segmentlength.Max = data.Length;
            if (segmentlength.Max > segmentlength.Value) {
                segmentlength.Value = segmentlength.Max;
            }
            this.values = data;
            compressionDone = false;                
        }

        public void CompressDataset()
        {
            CompressionFrameworkLib.CompressionUtility cu = new CompressionFrameworkLib.CompressionUtility();
            decompressed = cu.getDecompressedCHEBData(values, (float)epsilon.Value, (int)segmentlength.Value);
            compressionration = cu.getCompressionRatio();
            compressionDone = true;
        }

        public float[] GetCompressedDataset()
        {
            throw new NotImplementedException("ich bin halt faul =) ");
        }

        public float[] GetDecompressedDataset()
        {
            if (compressionDone)
                return decompressed;
            else
                throw new CompressionNotCalledException("");
        }


        public float getCompressionRatio()
        {
            if (compressionDone)
                return compressionration;
            else
                throw new CompressionNotCalledException("");
        }


        public Dictionary<string, RangedValue> GetOptimizationParameters()
        {
            return GetParameters();
        }

        public string Name
        {
            get { return "CHEB"; }
        }
    }
}
