﻿using RDotNet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Framework.compression.algorithms
{
    class PCA : ICompressionAlgorithm
    {
        public string Name = "PCA";
        
        REngine R;
        RangedIntValue initial = new RangedIntValue(1, 1, int.MaxValue);
        RangedFloatValue threshold = new RangedFloatValue(0.05f, 0, 0.3f);
        RangedIntValue final = new RangedIntValue(1, 1, int.MaxValue);
        RangedIntValue maxDegree = new RangedIntValue(2, 0, 5);
        RangedIntValue minDegree = new RangedIntValue(0, 0, 1);
        double[] rawDouble; // R nennt das "segment"
        float[] rawData;
        double[] decomp;

        public PCA()
        {
            REngine.SetEnvironmentVariables();
            R = REngine.GetInstance();

            // set default parameters //
            R.SetSymbol("initial", R.CreateInteger((int)initial.Value));
            R.SetSymbol("threshold", R.CreateNumeric((float)threshold.Value));
            R.SetSymbol("max_degree", R.CreateInteger((int)maxDegree.Value + 1));
            R.SetSymbol("min_degree", R.CreateInteger((int)minDegree.Value + 1));
        }

        // Es stehen zur Verfügung: 
        // initial = erster Datenpunkt im Array raw auf den die Kompression ausgeführt wird (erster Wert hat den Index 1)
        // final = letzter Datenpunkt im Array raw auf den die Kompression ausgeführt wird. final wird bei SetDataset automatisch auf den letzten Datenpunkt gesetzt
        // threshold = weiß man nicht so genau
        // maxDegree = höchster Polynomgrad der generiert wird
        // minDegree = kleinster Polynomgrad der generiert wird
        public void SetParameters(Dictionary<string, RangedValue> parameters)
        {
            foreach (KeyValuePair<string, RangedValue> kvp in parameters)
            {
                switch (kvp.Key)
                {
                    case "initial":
                        initial.Value = ((RangedIntValue)kvp.Value).Value;
                        break;
                    case "final":
                        final.Value = ((RangedIntValue)kvp.Value).Value;
                        break;
                    case "threshold":
                        threshold.Value = ((RangedFloatValue)kvp.Value).Value;
                        break;
                    case "maxDegree":
                        maxDegree.Value = ((RangedIntValue)kvp.Value).Value;
                        break;
                    case "minDegree":
                        minDegree.Value = ((RangedIntValue)kvp.Value).Value;
                        break;
                    default:
                        throw new InvalidOperationException("Parameter " + kvp.Key + " existiert in Algorithmus nicht");
                }
            }

            if (null != rawData)
            {
                float epsilon = Utilities.calculateAbsoluteEpsilon(rawData, threshold.Value);
                R.SetSymbol("threshold", R.CreateNumeric(epsilon));
            }            
            
            R.SetSymbol("initial", R.CreateInteger((int)initial.Value));            
            R.SetSymbol("final", R.CreateInteger((int)final.Value));
            R.SetSymbol("max_degree", R.CreateInteger((int)maxDegree.Value + 1));
            R.SetSymbol("min_degree", R.CreateInteger((int)minDegree.Value + 1));
        }

        public Dictionary<string, RangedValue> GetParameters()
        {
            Dictionary<string, RangedValue> parameters = new Dictionary<string, RangedValue>();
            //parameters.Add("initial", initial);
            //parameters.Add("final", final);
            parameters.Add("threshold", threshold);
            parameters.Add("maxDegree", maxDegree);
            parameters.Add("minDegree", minDegree);
            return parameters;
        }

        // paramenter final wird implizit mitgesetzt
        public void SetDataset(float[] data)
        {
            this.rawData = data;
            // recalculate epsilon
            float epsilon = Utilities.calculateAbsoluteEpsilon(rawData, threshold.Value);
            R.SetSymbol("threshold", R.CreateNumeric(epsilon));

            this.rawDouble = data.Select(x => (double)x).ToArray();
            R.SetSymbol("segment", R.CreateNumericVector(rawDouble));
            final.Value = rawDouble.Length;
            final.Max = rawDouble.Length;
            initial.Max = rawDouble.Length;
            R.SetSymbol("final", R.CreateInteger((int)final.Value));
        }

        public void CompressDataset()
        {
            R.Evaluate("source('PCALib/Linear_Seidel_Algo.R')");
            R.Evaluate("source('PCALib/Linear_Approx_Algo.R')");
            R.Evaluate("source('PCALib/multimodel_approximation_using_polynomials.R')");
            decomp = R.GetSymbol("decompData").AsNumeric().ToArray();
            //R.ClearGlobalEnvironment();
        }

        public float[] GetCompressedDataset()
        {
            throw new NotImplementedException();
        }

        public float[] GetDecompressedDataset()
        {
            // double[] to float[] Konvertierung
            float[] result = new float[decomp.Length];
            for (int i = 0; i < decomp.Length; i++)
            {
                result[i] = Convert.ToSingle(decomp[i]);
            }
            return result;
        }


        public float getCompressionRatio()
        {
            return 1 / Convert.ToSingle(R.GetSymbol("cr").AsNumeric().ToArray()[0]);
        }


        public Dictionary<string, RangedValue> GetOptimizationParameters()
        {
            return GetParameters();
        }

        string ICompressionAlgorithm.Name
        {
            get { return "PCA"; }
        }
    }
}
