﻿using System;
using System.Collections.Generic;

using Framework.changedetection;
using Framework.compression;


namespace Framework.optimization
{
    interface IOptimizationAlgorithm
    {
        /// <summary>
        /// Optimize the set of initialValues by minimizing the fitnessFunction.
        /// </summary>
        RangedValue[] Solve(Func<RangedValue[], FitnessObject> fitnessFunction, RangedValue[] initialValues);

        /// <summary>
        /// Sets the parameters for the algorithm. Since each algorithm has its own parameters a string-float dictionary is used.
        /// </summary>
        /// <param name="parameters">A dictionary of parameter names and values.</param>
        void SetParameters(Dictionary<string, RangedValue> parameters);
        /// <summary>
        /// Gets the parameters for the algorithm. Since each algorithm has its own parameters a string-float dictionary is used.
        /// </summary>
        Dictionary<string, RangedValue> GetParameters();

        void SetParamNames(List<string> paramNames);

        List<DeutscheBahn.OptimizationResult> getOptimizationResults();
    }
}
